/*(C) Umang Gajera - www.ocfreaks.com
More Embedded tutorials @ www.ocfreaks.com/cat/embedded/
LPC2148 ADC Example 3 Source code.
@ http://www.ocfreaks.com/lpc2148-analog-to-digital-conversion-adc-programming-tutorial/
License : GPL.*/

#include <lpc214x.h>
#include "ocfreaks_sh.h" //OCFreaks LPC214x Tutorial Support Header

#define AD06 ((1<<9)|(1<<8)) //Select AD0.6 function for P0.4
#define AD07 ((1<<11)|(1<<10)) //Select AD0.7 function for P0.5

#define ADINTEN6 (1<<6) //Interrupt Enable for Channel 6
#define ADINTEN7 (1<<7) //Interrupt Enable for Channel 7
#define SEL_AD06 (1<<6) //Select Channel 6  for A/D Conversion
#define SEL_AD07 (1<<7) //Select Channel 7 for A/D Conversion

#define CLKDIV (15-1) //4Mhz ADC clock (ADC_CLOCK=PCLK/CLKDIV) where "CLKDIV-1" is actually used , in our case PCLK=60mhz  
#define BURST_MODE_ON (1<<16) //1 for on and 0 for off
#define CLKS_10bit ((0<<19)|(0<<18)|(0<<17)) //10bit - used in burst mode only
#define PowerUP (1<<21)
#define VREF 3.3 //Reference Voltage at VREF Pin

void initClocks(void);
void initTimer0(void);
__irq void AD0ISR(void);

void setupPLL0(void);
void feedSeq(void);
void connectPLL0(void);

int AD06Result=0 , AD07Result=0; //Global Variables

int main(void)
{
	initClocks(); //CCLK and PCLK @ 60Mhz - defined in ocfreaks_sh.h
	initUART0(); //Initialize UART0 for printf - defined in ocfreaks_sh.h
	
	PINSEL0 |= AD06 | AD07 ; //Select AD0.6 and AD0.7 for P0.4 and P0.5 respectively
	
	VICIntEnable |= (1<<18) ;
	VICVectCntl0 = (1<<5) | 18 ;
	VICVectAddr0 = (unsigned) AD0ISR;
	
	printf("OCFreaks.com ADC Tutorial Example 3.\nHardware Controlled Mode(BURST Mode) ADC on AD06 & AD07 Channel.\n");
	
	unsigned long AD0CR_setup = (CLKDIV<<8) | BURST_MODE_ON | CLKS_10bit | PowerUP; //Setup ADC
	AD0INTEN = ADINTEN6 | ADINTEN7 ; //Enable AD06 and AD07 Interrupts 
	AD0CR =  AD0CR_setup | SEL_AD06 | SEL_AD07; 
	
	delayMS(10); //Wait till initial conversions get finished
	
	float volt1=0,volt2=0;
	
	while(1)
	{
		volt1 = ((float)AD06Result*VREF)/1024; //Convert AD0.6 result to volts.
		volt2 = ((float)AD07Result*VREF)/1024; //Convert AD0.7 result to volts.
		printf("AD06 = %dmV | AD07 = %dmV\n",(int)(volt1*1000),(int)(volt2*1000)); //Display millivolts
		delayMS(250); //4 updates per second
	}
	
	//return 0; //this wont execute normally
}

__irq void AD0ISR(void) //AD0 Interrupt Function
{
	unsigned long dummyRead;
	unsigned long AD0GDR_Read = AD0GDR;
	
	int channel = (AD0GDR_Read>>24) & 0xF; //Extract Channel Number
	int currentResult = (AD0GDR_Read>>6) & 0x3FF; //Extract Conversion Result

	if(channel == 6)
	{ 
		dummyRead = AD0DR6; //Read to Clear Done flag , Also clears AD0 interrupt
		AD06Result = currentResult;
	}
	else if(channel == 7)
	{
		dummyRead = AD0DR7; //Read to Clear Done flag , Also clears AD0 interrupt
		AD07Result = currentResult;
	}
	
	VICVectAddr = 0x0; //Signal that ISR has finished
}
